% Perform 3D+t Mathematical Morphology operations on a given channel
%
%    <CustomTools>
%      <Menu name="Bioimaging XT">
%       <Submenu name="3D+t">
%        <Item name="Mathematical Morphology" icon="Matlab" tooltip="Mathematical Morphology">
%          <Command>MatlabXT::XTB_xyzt_MM(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%    </CustomTools>
%
% Nicolas Liaudet - Bioimaging Core Facility - Jui 2018
% v1. 15.07.18 Nicolas Liaudet
%     30.07.18
%              Apply button uses the original image and not the current one
%              NL    




function [ output_args ] = XTB_xyzt_MM(aImarisApplicationID)
%XTB_xyzt_MM
%   3D mathematical morphology
%    <CustomTools>
%      <Menu>
%        <Item name="Mathematical Morphology" icon="Matlab" tooltip="Mathematical Morphology">
%          <Command>MatlabXT::XTB_xyzt_MM(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
disp('_____________________________________________________')
disp('3D mathematical morphology')
disp('Jui 2018, Bioimaging Core Facility')
disp(' v1. Nicolas Liaudet')
disp('_____________________________________________________')

% connect to Imaris interface
disp('Imaris connection...')
tic
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
    vImarisApplication = aImarisApplicationID;
end
% vDataSet = vImarisApplication.GetDataSet;%deprecated
vDataSet = vImarisApplication.GetImage(0);
toc

vDataSizeYXZTC = [vDataSet.GetSizeX, vDataSet.GetSizeY,...
                  vDataSet.GetSizeZ, vDataSet.GetSizeT,...
                  vDataSet.GetSizeC];

% get the channel names
ChName = cell(vDataSizeYXZTC(5),1);
for idxC = 1:vDataSizeYXZTC(5)
    ChName{idxC} = char(vDataSet.GetChannelName(idxC-1));
end

switch char(vDataSet.GetType)
    case 'eTypeUInt8'
        BitDepth = 'uint8';
    case 'eTypeUInt16'
        BitDepth = 'uint16';
    case 'eTypeFloat'
        BitDepth = 'single';
end


uiselect(ChName,BitDepth,vDataSizeYXZTC,vDataSet,vImarisApplication)
end
 
function uiselect(ChName,BitDepth,vDataSizeYXZTC,vDataSet,vImarisApplication)
%GUI for user interaction


h.fig = figure('units','pixels','position',[50 50 400 150],...
    'toolbar','none','menu','none','Name','Mathematical Morphology',...
    'Resize','off','NumberTitle','off','Visible','off');
%,'CloseRequestFcn','');

dCh(:,1) = repmat({true},[size(ChName)]);
dCh(:,2) = ChName;

h.t = uitable('Parent',h.fig,'Data',dCh,...
    'ColumnEditable',[true false],...
    'Units','Pixel','Position', [10 20 119 110],...
    'ColumnWidth',{20,80},...
    'RowName',{},'ColumnName',{});


uicontrol('Parent',h.fig,'Style','text','Units','pixel',...
    'Position',[137 112 20 13],'String','Do:',...
    'HorizontalAlignment','Left');
h.MMop = uicontrol('Parent',h.fig,'Style','popup','Units','pixel',...
    'String', {'Erosion','Dilation','Opening','Closing',...
    'White top-hat','Black top-hat'},...
    'Position', [160 115 100 15]);


h.p = uipanel('Parent',h.fig,'Units','Pixel',...
    'Position',[137 20 200 85],'TitlePosition','CenterTop',...
    'title','Structuring element');
ds = 15;
h.txtshape = uicontrol('Parent',h.p,'Style','text','Units','pixel',...
    'Position',[5 2+3*ds 40 13],'String','Shape:',...
    'HorizontalAlignment','Left');
h.txtDim1 = uicontrol('Parent',h.p,'Style','text','Units','pixel',...
    'Position',[5 -5+2*ds 50 13],'String','Radius:',...
    'HorizontalAlignment','Left');
h.txtDim2 = uicontrol('Parent',h.p,'Style','text','Units','pixel',...
    'Position',[5 -10+1*ds 65 13],'String','Periodic-line:',...
    'HorizontalAlignment','Left',...
    'Visible','off');
h.txtDim3 = uicontrol('Parent',h.p,'Style','text','Units','pixel',...
    'Position',[133,5,35,15],'String','height:',...
    'HorizontalAlignment','Left',...
    'Visible','off');



h.SEShape = uicontrol('Parent',h.p,'Style','popup','Units','pixel',...
    'String', {'disk','diamond','line',...
    'octagon','rectangle','square','cube','cuboid','sphere'},...
    'Position', [50 5+3*ds 80 15],'Value',9,...
    'Callback', @SHAPEPara);

h.Space = uicontrol('Parent',h.p,'Style','popup','Units','pixel',...
    'String', {'3D','XY','XZ','YZ'},...
    'Position', [137,54,54,11]);


h.Dim1 = uicontrol('Parent',h.p,'Style','edit','Units','pixel',...
    'Position',[90 -5+2*ds 40 13],'String',num2str(4),'TooltipString','In pixels');

h.Dim2 = uicontrol('Parent',h.p,'Style','edit','Units','pixel',...
    'Position',[90 -10+1*ds 40 13],'String',num2str(0),'TooltipString','0, 4, 6, or 8',...
    'Visible','off');

h.Dim3 = uicontrol('Parent',h.p,'Style','edit','Units','pixel',...
    'Position',[167,7,28,13],'String',num2str(0),'TooltipString','0, 4, 6, or 8',...
    'Visible','off');

h.but(1) = uicontrol('Parent',h.fig,'Style','PushButton','String','APPLY',...
    'Units','pixel','Position',[344 106 50 25],'FontWeight','Bold',...
    'Callback',@APPLYT);

h.but(2) = uicontrol('Parent',h.fig,'Style','PushButton','String','OK',...
    'Units','pixel','Position',[344 45 50 25],'FontWeight','Bold',...
    'Callback',@OK);

h.but(3) = uicontrol('Parent',h.fig,'Style','PushButton','String','CANCEL',...
    'Units','pixel','Position',[344 20 50 25],'FontWeight','Bold',...
    'Callback',@CANCEL);

align(h.but,'VerticalAlignment','Distribute')


movegui(h.fig,'center')


% vDataSet = vImarisApplication.GetDataSet;%deprecated
vDataSet = vImarisApplication.GetImage(0);

vDataSetOriginal = vDataSet.Clone;
OriginalStacksXYZTC = zeros(vDataSizeYXZTC,BitDepth);
switch BitDepth
    case 'uint8'
        GetFun = @vDataSetOriginal.GetDataVolumeAs1DArrayBytes;
    case 'uint16'
        GetFun = @vDataSetOriginal.GetDataVolumeAs1DArrayShorts;
    case 'single'
        GetFun = @vDataSetOriginal.GetDataVolumeAs1DArrayFloats;
end
for idxC = 1:vDataSizeYXZTC(5)
    for idxT = 1:vDataSizeYXZTC(4)
        im = typecast(GetFun(idxC-1,idxT-1),BitDepth);        
        im = reshape(im,vDataSizeYXZTC([1 2 3]));
        OriginalStacksXYZTC(:,:,:,idxT,idxC) = im;
    end
end
set(h.fig,'Visible','on')
        
        
        
    function SHAPEPara(hObject,eventdata)
        txtSH = get(h.SEShape,'String');
        txtSH = txtSH{get(h.SEShape,'Value')};
        
        switch txtSH
            case 'disk'
                set(h.txtDim1,'String','Radius:')
                set(h.Dim1,'TooltipString','In pixels')
                set(h.txtDim2,'String','Periodic-line:','Visible','on')
                set(h.Dim2,'String','0','Visible','on','TooltipString','0, 4, 6, or 8')
                set(h.txtDim3,'String','height:','Visible','off')
                set(h.Dim3,'String','3','Visible','off','TooltipString','In pixels')
            case 'diamond'
                set(h.txtDim1,'String','Radius:')
                set(h.Dim1,'TooltipString','In pixels')
                set(h.txtDim2,'String','PeriodL','Visible','off')
                set(h.Dim2,'Visible','off','TooltipString','0, 4, 6, or 8')
                set(h.txtDim3,'String','height:','Visible','off')
                set(h.Dim3,'String','3','Visible','off','TooltipString','In pixels')
            case 'line'
                set(h.txtDim1,'String','Length:')
                set(h.Dim1,'TooltipString','In pixels')
                set(h.txtDim2,'String','Angle:','Visible','on')
                set(h.Dim2,'Visible','on','TooltipString','In degrees')
                set(h.txtDim3,'String','height:','Visible','off')
                set(h.Dim3,'String','3','Visible','off','TooltipString','In pixels')
            case 'octagon'
                set(h.txtDim1,'String','Radius:')
                set(h.Dim1,'String','3','TooltipString','In pixel, must be a nonnegative multiple of 3')
                set(h.txtDim2,'String','PeriodL','Visible','off')
                set(h.Dim2,'Visible','off','TooltipString','0, 4, 6, or 8')
                set(h.txtDim3,'String','height:','Visible','off')
                set(h.Dim3,'String','3','Visible','off','TooltipString','In pixels')            
            case 'rectangle'
                set(h.txtDim1,'String','Row:')
                set(h.Dim1,'TooltipString','In pixels')
                set(h.txtDim2,'String','Column:','Visible','on')
                set(h.Dim2,'String','3','Visible','on','TooltipString','In pixels')
                set(h.txtDim3,'String','height:','Visible','off')
                set(h.Dim3,'String','3','Visible','off','TooltipString','In pixels')
            case 'square'
                set(h.txtDim1,'String','Width:')
                set(h.Dim1,'TooltipString','In pixels')
                set(h.txtDim2,'String','PeriodL','Visible','off')
                set(h.Dim2,'Visible','off','TooltipString','0, 4, 6, or 8')
                set(h.txtDim3,'String','height:','Visible','off')
                set(h.Dim3,'String','3','Visible','off','TooltipString','In pixels')
            case 'cube'
                set(h.txtDim1,'String','Width:')
                set(h.Dim1,'TooltipString','In pixels')
                set(h.txtDim2,'String','PeriodL','Visible','off')
                set(h.Dim2,'Visible','off','TooltipString','In pixels')
                set(h.txtDim3,'String','height:','Visible','off')
                set(h.Dim3,'String','3','Visible','off','TooltipString','In pixels')
            case 'cuboid'
                set(h.txtDim1,'String','Row:')
                set(h.Dim1,'TooltipString','In pixels')
                set(h.txtDim2,'String','Column:','Visible','on')
                set(h.Dim2,'Visible','on','TooltipString','In pixels')
                set(h.txtDim3,'String','Height:','Visible','on')
                set(h.Dim3,'String','3','Visible','on','TooltipString','In pixels')
            case 'sphere'
                set(h.txtDim1,'String','Radius:')
                set(h.Dim1,'TooltipString','In pixels')
                set(h.txtDim2,'String','PeriodL','Visible','off')
                set(h.Dim2,'Visible','off','TooltipString','In pixels')
                set(h.txtDim3,'String','height:','Visible','off')
                set(h.Dim3,'String','3','Visible','off','TooltipString','In pixels')
        end
    end

    function APPLYT(hObject,eventdata)
        %function for the translation and display
        
        txtOP = get(h.MMop,'String');
        txtOP = txtOP{get(h.MMop,'Value')};
        txtSH = get(h.SEShape,'String');
        txtSH = txtSH{get(h.SEShape,'Value')};
        dim1 = round(str2double(get(h.Dim1,'String')));
        dim2 = str2num(get(h.Dim2,'String'));
        dim3 = str2num(get(h.Dim3,'String'));
        
        if ismember(txtSH,{'diamond','octagon','square','cube','sphere'})
            Bc = strel(txtSH,dim1);
        end
        if ismember(txtSH,{'disk','line'})
            Bc = strel(txtSH,dim1,dim2);
        end
        if ismember(txtSH,{'rectangle'})
            Bc = strel(txtSH,[dim1,dim2]);
        end
        if ismember(txtSH,{'cuboid'})
            Bc = strel(txtSH,[dim1,dim2,dim3]);
        end        
        Bc = Bc.Neighborhood;
        switch  h.Space.String{h.Space.Value}
            case '3D'
                Bc = strel(Bc);
            case 'XY'
                Bc = strel(Bc);
            case 'XZ'                
                Bc = strel(permute(Bc,[3 2 1]));           
            case 'YZ'                
                Bc = strel(permute(Bc,[1 3 2]));
        end
        
        switch txtOP
            case 'Dilation'
                m_fun = @imdilate;
            case 'Erosion'
                m_fun = @imerode;
            case 'Opening'
                m_fun = @imopen;
            case 'Closing'
                m_fun = @imclose;
            case 'White top-hat'
                m_fun = @imtophat;
            case 'Black top-hat'
                m_fun = @imbothat;
        end
        chsel = get(h.t,'DATA');
        chsel = cell2mat(chsel(:,1));
        
        hwbar = waitbar(0, ['Processing...']);
        vDataSetClone = vDataSet.Clone;
        switch BitDepth
            case 'uint8'
                SetFun = @vDataSetClone.SetDataVolumeAs1DArrayBytes;
                Dyn = [0 intmax(BitDepth)];
            case 'uint16'
                SetFun = @vDataSetClone.SetDataVolumeAs1DArrayShorts;
                Dyn = [0 intmax(BitDepth)];
            case 'single'
                SetFun = @vDataSetClone.SetDataVolumeAs1DArrayFloats;
                Dyn = [0 1];
        end
        cnt = 1;
        for idxC = 1:vDataSizeYXZTC(5)
            if chsel(idxC)
                for idxT = 1:vDataSizeYXZTC(4)
                    hwbar = waitbar(cnt/(vDataSizeYXZTC(4)*sum(chsel)), hwbar,...
                        ['Processing ' ChName{idxC} ' -> time frame ' num2str(idxT) '/' num2str(vDataSizeYXZTC(4))]);
                    im = OriginalStacksXYZTC(:,:,:,idxT,idxC);
                    im = m_fun(im,Bc);                    
                    SetFun(im(:),idxC-1,idxT-1);
                    cnt = cnt+1;
                end                
            end
        end
        close(hwbar)
        vImarisApplication.SetImage(0, vDataSetClone)                           
        
    end

    function OK(hObject,eventdata)
        close(h.fig)
    end

    function CANCEL(hObject,eventdata)
       chsel = get(h.t,'DATA');
        chsel = cell2mat(chsel(:,1));        
        vDataSetClone = vDataSet.Clone;
        hwbar = waitbar(0, ['Processing...']);
        switch BitDepth
            case 'uint8'
                SetFun = @vDataSetClone.SetDataVolumeAs1DArrayBytes;
                Dyn = [0 intmax(BitDepth)];
            case 'uint16'
                SetFun = @vDataSetClone.SetDataVolumeAs1DArrayShorts;
                Dyn = [0 intmax(BitDepth)];
            case 'single'
                SetFun = @vDataSetClone.SetDataVolumeAs1DArrayFloats;
                Dyn = [0 1];
        end
        cnt = 1;
        for idxC = 1:vDataSizeYXZTC(5)
            if chsel(idxC)
                for idxT = 1:vDataSizeYXZTC(4)
                    hwbar = waitbar(cnt/(vDataSizeYXZTC(4)*sum(chsel)), hwbar,...
                        ['Processing ' ChName{idxC} ' -> time frame ' num2str(idxT) '/' num2str(vDataSizeYXZTC(4))]);
                    BW = OriginalStacksXYZTC(:,:,:,idxT,idxC);
                     SetFun(BW(:),idxC-1,idxT-1);
                    cnt = cnt+1;
                end
                vDataSetClone.SetChannelRange(idxC-1,Dyn(1),Dyn(2))
            end
        end
        
        close(hwbar)
        vImarisApplication.SetImage(0, vDataSetClone)
        close(h.fig)
    end

end


